from frictionless import Resource, Pipeline, steps


# General


def test_resource_transform():
    source = Resource(path="data/transform.csv")
    pipeline = Pipeline(
        steps=[
            steps.table_normalize(),
        ],
    )
    target = source.transform(pipeline)
    assert target.schema.to_descriptor() == {
        "fields": [
            {"name": "id", "type": "integer"},
            {"name": "name", "type": "string"},
            {"name": "population", "type": "integer"},
        ]
    }
    assert target.read_rows() == [
        {"id": 1, "name": "germany", "population": 83},
        {"id": 2, "name": "france", "population": 66},
        {"id": 3, "name": "spain", "population": 47},
    ]


def test_resource_transform_cell_set():
    source = Resource("data/transform.csv")
    pipeline = Pipeline.from_descriptor(
        {
            "steps": [
                {"type": "cell-set", "fieldName": "population", "value": 100},
            ],
        }
    )
    target = source.transform(pipeline)
    assert target.schema.to_descriptor() == {
        "fields": [
            {"name": "id", "type": "integer"},
            {"name": "name", "type": "string"},
            {"name": "population", "type": "integer"},
        ]
    }
    assert target.read_rows() == [
        {"id": 1, "name": "germany", "population": 100},
        {"id": 2, "name": "france", "population": 100},
        {"id": 3, "name": "spain", "population": 100},
    ]
